///////////////////////////////////////////////////////
// Example: Computation of the canonical lift of the //
//          van Wamelen curve.                       //
///////////////////////////////////////////////////////

/* The Van Wamelen curve has been suggested to me as example
   by J. Scholten */

Attach("thetanull_genus2.m");

FF := FiniteField(3);
P<x> := PolynomialRing(FF);
f := 52*x^5-156*x^4+208*x^3-156*x^2+64*x-11;
H := HyperellipticCurve(f);
print "----------------------------------------------------------------------";
printf "Input is a ";
print H;
print "----------------------------------------------------------------------";

T := ThomaeConstants(H);
printf "The Thomae constants are defined over ";
print BaseRing(Parent(T));
TT := CompleteFourThetaNullPointOverField(T);
printf "The full 4-theta null point is defined over ";
print BaseRing(Parent(TT));
print "----------------------------------------------------------------------";

print "Lifting the Thomae constants to precision 200.";
time TL2 := CanonicalLiftTwoThetaNullPoint(T,200);
// print "The lifted theta null point equals ", TL2; 
print "----------------------------------------------------------------------";

print "Let [1,a02,a20,a22] be the 2-torsion part of the 4-theta null point.";
a00, a02, a20, a22 := Explode(Eltseq(TL2));
PZ<x> := PolynomialRing(Integers());
// The following minimal polynomial has been computed with code by D. Kohel
MinPol20 := x^20-5*x^19+23*x^18-53*x^17+112*x^16-203*x^15+279*x^14-345*x^13+360*x^12-333*x^11+329*x^10-333*x^9+360*x^8-345*x^7+279*x^6-203*x^5+112*x^4-53*x^3+23*x^2-5*x+1;
assert IsIrreducible(MinPol20);
assert (Evaluate(MinPol20,a20) eq 0);
printf "The minimal polynomial of a20 is given by ";
print MinPol20;
print "----------------------------------------------------------------------";

PQ<t> := PolynomialRing(Rationals());
J := Jacobian(H);
E<t> := NumberField(EulerFactor(J));
printf "The endomorphism algebra E of the Jacobian variety of the above hyperelliptic curve equals the the number field generated by the polynomial ";
print MinimalPolynomial(t);
print "----------------------------------------------------------------------";

printf "This number field has class number ";
printf "%m", ClassNumber(E);
printf " and Galois group equal to a ";
print GaloisGroup(E);
print "----------------------------------------------------------------------";

PE<z> := PolynomialRing(E);
NewPol20 := PE!MinPol20;
factor := Factorisation(NewPol20)[1][1];
N := ext<E|factor>;
printf "The root a20 generates over the number field E the extension ";
print N;
print "----------------------------------------------------------------------";

printf "The latter field is an abelian extension with conductor given by the ";
print Conductor(AbelianExtension(N));
print "----------------------------------------------------------------------";

O := MaximalOrder(E);
I := 2 * O;
printf "The ray class field for the ideal [2,0,0,0] has degree ";
printf "%m", Degree(RayClassField(I));
print " and hence coincides with the latter abelian extension.";
